/*
 * IBM Technical Support Appliance Version 2 Release 8 Modification 0
 *
 * (C) Copyright IBM Corp. 2017, 2021
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/VerificationToolkit'
  , 'tsa/cmd/Command'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  ],
  
  function( $
          , TV
          , VT
          , Command
          , AbstractHandler
          , ButtonFactory
          , ContentArea
          , FormPanel
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'ScopesettingsHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.ScopesettingsHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.ScopesettingsHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.ScopesettingsHandler.prototype ).constructor = tsa.cmd.handlers.ScopesettingsHandler;
    
    //-------------------------------------------------------------------------
    /**
     * The maximum recommended number of scope entries.
     * @private
     * @type {number}
     */
    tsa.cmd.handlers.ScopesettingsHandler.MAX_RECOMMENDED_ENTRIES = 400;
    
    /**
     * The content area widget instance.
     * @private
     * @type {tsa.widgets.ContentArea}
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.contentArea = null;
    
    /**
     * Instructions message.
     * @private
     * @type {Element}
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.instructionsMsg = null;
    
    /**
     * The form panel widget instance.
     * @private
     * @type {tsa.widgets.FormPanel}
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.formPanel = null;
    
    /**
     * The HTML Input for the scope name.
     * @private
     * @type {Element}
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.nameInput = null;
    
    /**
     * The HTML Input for the file containing scope entries.
     * @private
     * @type {Element}
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.fileInput = null;
    
    /**
     * The map of file entries and their attributes.
     * @private
     * @type {Object.<string,{valid:boolean
     *                       ,count:number}>}
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.fileEntries = null;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype._index = function()
    {
      /** @type {Object} */ var i18n = tsa.i18n;
      
      this.contentArea = new ContentArea ( { title       : i18n.labelImportScopeHeading
                                           , helpDoc     : 'sshelp_scopeset_settings_panel.html' } );
      this.formPanel   = new FormPanel   ( { title       : i18n.labelImportScopeSetHeading
                                           , description : i18n.labelImportScopeSetDesc } );
      
      this.nameInput = $( '<input type="text" name="name" id="name"    size="50" maxlength="250" required/>' ).get( 0 );
      this.fileInput = $( '<input type="file" name="fileUpload" id="fileUpload" required/>' ).get(0);
      
      // Clear previous message
      $( this.fileInput ).on( 'change', function(){tsa.clearSysMessages();});
      
      this.formPanel.appendFieldRow( { label    : i18n.labelImportScopeSetNameHeading
                                     , required : true
                                     , element  : this.nameInput } );
      this.formPanel.appendFieldRow( { label    : i18n.labelImportScopeSet
                                     , required : true
                                      , element  : this.fileInput 
                                      , hint : i18n.labelImportScopeSetNameDesc} );
      
      this.formPanel.appendButton( { type    : 'arrow'
                                   , label   : i18n.buttonImportScopeSet
                                   , onclick : $.proxy( this.onImportButtonClicked, this ) } );
      
      this.instructionsMsg = this.contentArea.appendMessage( { text : i18n.labelImportScopeSetCaption
                                                                    + '\n\n'
                                                                    + i18n.labelImportScopeSetLabelCaption } );
      this.contentArea.append( this.formPanel.getDomNode() );
      
      this.show( this.contentArea.getDomNode() );
    };
    
    //-------------------------------------------------------------------------
    /**
     * Load file entries.
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.loadFileEntries = function() {

        this.fileEntries = {};
      
        if (!TV.typeIsUndefined(typeof FileReader          ) &&
            !TV.typeIsUndefined(typeof this.fileInput.files)) {
            if (this.fileInput.files.length > 0) {
                /** @type {Array.<string>} */ var valid = [];

                /** @type {number} */ var blankLines = 0;

                /** @type {FileReader} */ var fileReader = new FileReader();
          
                fileReader.onload = $.proxy(function(progressEvent) {
                    /** @type {Array.<string>} */ var lines      = fileReader.result.split( '\n' );
                    /** @type {number}         */ var blankLines = 0;
            
                    $.each(lines, $.proxy(function(lineIndex, line) {
                        if ((line = $.trim(line)).length > 0) {
                            valid.push(line);
                        } else {
                            blankLines++;
                        }
                    }
                    , this ) );
            
                    logger.info('File entries loaded. Results:'    + '\n' + 
                                '    Total lines: ' + lines.length + '\n' + 
                                '    Blank lines: ' + blankLines);
            
                    // Send list to TSA server to validate entries......
                    /** @type {tsa.cmd.Command} */ var cmd = new Command( { action     : 'scopesettings/ajaxValidateImportScopeSet',
                                                                            parameters : { 'entries' : valid.join( ' ' ) }
                                                                          } );
            
                    cmd.execute();    
            
                    $(cmd).on('executionSucceeded', $.proxy(function(event) {
            	        var validEntries          = event.response.payload.validEntries;
            	        var duplicateEntries      = event.response.payload.duplicateEntries;
             	        var invalidEntries        = event.response.payload.invalidEntries;
            	        var unresolvedFqdnEntries = event.response.payload.unresolvedFqdnEntries;

                        logger.info( 'File entries validated. Results:'                         + '\n' + 
                                     '    valid entries'              + ': ' + validEntries     + '\n' + 
                                     '    duplicate entries'          + ': ' + duplicateEntries + '\n' + 
                                     '    invalid entries'            + ': ' + invalidEntries   + '\n' + 
                                     '    unresolved FQDN entries'    + ': ' + unresolvedFqdnEntries );

                        // Show the confirmation dialog.  Simple dialog is displayed if all entries are valid, 
                        // otherwise a more in depth dialog is shown with errors in the list shown.
                        this.showConfirmationDialog(validEntries, 
                	                                duplicateEntries, 
                		                            invalidEntries, 
                		                            unresolvedFqdnEntries);
            
                     }
                     , this ) );
          
                }
                , this );
          
                // Reading using default encoding: utf-8
                fileReader.readAsText( this.fileInput.files[0] );
        
            } else {
                logger.warn( 'File entries can\'t be loaded: file not selected.' );
            }
      
        } else {
            logger.error( 'Cannot read file: unsupported browser.' );
        }
    };
    
    //-------------------------------------------------------------------------
    /**
     * Import Button: click event handler.
     * @param {Event} event Event
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.onImportButtonClicked = function( event )
    {
      if( this.formPanel.validate() )
      {
        this.loadFileEntries();
      }
    };
    
    //-------------------------------------------------------------------------
    /**
     * Show confirmation dialog after TSA server analyzes the entries.
     * If problems were detected within the entry file, they will be displayed here.
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.showConfirmationDialog = function(validEntries, 
                                                                                      duplicateEntries, 
                                                                                      invalidEntries, 
                                                                                      unresolvedFqdnEntries) {
        /** @type {boolean} */ var tooMany   = false;
        /** @type {boolean} */ var noneValid = false;
        
        /** @type {Array.<string>} */ var valid      = [];
        /** @type {Array.<string>} */ var duplicate  = [];
        /** @type {Array.<string>} */ var invalid    = [];
        /** @type {Array.<string>} */ var unresolved = [];

        // Turn entries lists into arrays if any entries exist.
        if (validEntries.length > 0) {
            valid = validEntries.split( ',' );
        }
        if (duplicateEntries.length > 0) {
            duplicate = duplicateEntries.split( ',' );
        }
        if (invalidEntries.length > 0) {
        	invalid = invalidEntries.split( ',' );
        }
        if (unresolvedFqdnEntries.length > 0) {
            unresolved = unresolvedFqdnEntries.split( ',' );
        }
                                                                                      
        if (noneValid = (valid.length === 0)) {
            // Special case - none of the entries are valid.
            alert( tsa.i18n.labelImportScopeSetProblemNoneValid );
        
        } else if ((tooMany = (valid.length > tsa.cmd.handlers.ScopesettingsHandler.MAX_RECOMMENDED_ENTRIES)) ||
                   (invalid   .length > 0 ) ||
                   (duplicate .length > 0 ) ||
                   (unresolved.length > 0 )) {
            /** @type {Element} */ var problemsMsgNode   = document.createElement( 'h3' );
            /** @type {Element} */ var tooManyMsgNode    = null;
            /** @type {Element} */ var invalidMsgNode    = null;
            /** @type {Element} */ var duplicateMsgNode  = null;
            /** @type {Element} */ var unresolvedMsgNode = null;
            /** @type {Element} */ var validMsgNode      = null;
            /** @type {Element} */ var unresolvedMsgNode = null;
            /** @type {Element} */ var confirmMsgNode    = document.createElement( 'h3' );
          
            /** @type {string}  */ var tooManyMsgText    = tsa.bind( tsa.i18n.labelImportScopeSetStatus, tsa.cmd.handlers.ScopesettingsHandler.MAX_RECOMMENDED_ENTRIES ) ;
            /** @type {string}  */ var invalidMsgText    = tsa.i18n.labelImportScopeSetProblemInvalid;
            /** @type {string}  */ var duplicateMsgText  = tsa.i18n.labelImportScopeSetProblemValidDup;
            /** @type {string}  */ var validMsgText      = tsa.i18n.labelImportScopeSetValidCount;
            /** @type {string}  */ var unresolvedMsgText = tsa.i18n.labelImportScopeSetProblemResolve;
            /** @type {Element} */ var buttonContainer   = ButtonFactory.buildButtonContainer();
            /** @type {string}  */ var bullet            = String.fromCharCode( 0x2022 );
          
            $( this.instructionsMsg ).hide();
            $( this.formPanel.getDomNode() ).hide();
          
            $.each(invalid   , function(i, entry) {invalidMsgText    += '\n' + bullet + ' ' + entry;} );
            $.each(duplicate , function(i, entry) {duplicateMsgText  += '\n' + bullet + ' ' + entry;} );
            $.each(unresolved, function(i, entry) {unresolvedMsgText += '\n' + bullet + ' ' + entry;} );
          
            validMsgText += ' ' + (valid.length);
          
            $(problemsMsgNode).text(tsa.i18n.labelImportScopeSetProblemsFound);
            if (tooMany) {
                $(confirmMsgNode).text(tsa.i18n.labelImportScopeSetConfirmLimit);
            } else {
                $( confirmMsgNode  ).text( tsa.i18n.labelImportScopeSetConfirm );
            }
            this.contentArea.append( problemsMsgNode );
          
            if (tooMany              ) tooManyMsgNode    = this.contentArea.appendMessage( { level : 'WARN', border : false, text : tooManyMsgText    } );
            if (invalid   .length > 0) invalidMsgNode    = this.contentArea.appendMessage( { level : 'WARN', border : false, text : invalidMsgText    } );
            if (duplicate .length > 0) duplicateMsgNode  = this.contentArea.appendMessage( { level : 'WARN', border : false, text : duplicateMsgText  } );
            if (unresolved.length > 0) unresolvedMsgNode = this.contentArea.appendMessage( { level : 'WARN', border : false, text : unresolvedMsgText } );
          
            var closeDialog = $.proxy(function(event) {
                if (tooManyMsgNode    !== null) $(tooManyMsgNode   ).remove();
                if (invalidMsgNode    !== null) $(invalidMsgNode   ).remove();
                if (duplicateMsgNode  !== null) $(duplicateMsgNode ).remove();
                if (unresolvedMsgNode !== null) $(unresolvedMsgNode).remove();
            
                $(problemsMsgNode).remove();
                $(confirmMsgNode ).remove();
                $(validMsgNode   ).remove();
                $(buttonContainer).remove();
            
                $( this.instructionsMsg ).show();
                $( this.formPanel.getDomNode() ).show();
                $( this.fileInput ).val( '' );
            }
            , this );
            
            /** @type {Element} */ var onAccept = null; 
            /** @type {Element} */ var onCancel = null;
          
            onAccept = $.proxy( function( event ) {
                closeDialog();
                this.importScopeSet( $( this.nameInput ).val(), valid );
            }, this );
          
            onCancel = $.proxy(function(event) {
                closeDialog();
            }, this );
          
            $(buttonContainer).append(ButtonFactory.buildButton( { type : 'cancel', label : tsa.i18n.buttonCancel, onclick : onCancel } ) );
            $(buttonContainer).append(ButtonFactory.buildButtonSeparator() );
         
            $(buttonContainer).append(ButtonFactory.buildButton( { type : 'arrow', label : tsa.i18n.buttonAccept, onclick : onAccept } ) );
          
            this.contentArea.append( confirmMsgNode );
            validMsgNode = this.contentArea.appendMessage( { level : 'INFO', border: false, text : validMsgText } );
            this.contentArea.append( buttonContainer );
        
        } else {
            // A simple dialog is enough.  No problems found with import list.
            if (window.confirm(tsa.i18n.labelImportScopeSetConfirm)) {
                this.importScopeSet( $( this.nameInput ).val(), valid );
            }
        }
    };
    
    //-------------------------------------------------------------------------
    /**
     * Import Scope Set: perform the server request.
     * @private
     * @param {string}         name    The scope name.
     * @param {Array.<string>} entries The entries, previously validated IPv4 addresses.
     */
    tsa.cmd.handlers.ScopesettingsHandler.prototype.importScopeSet = function( name, entries )
    {
      /** @type {tsa.cmd.Command} */ var cmd = new Command( { action     : 'scopesettings/ajaxImportScopeSet'
                                                            , parameters : { 'name'    : name
                                                                           , 'entries' : entries.join( ' ' ) }
                                                            } );
      $( cmd ).on( 'executionSucceeded', $.proxy( function( event )
      {
        $( this.nameInput ).val( '' );
      }
      , this ) );
      
      $( cmd ).on( 'executionFailed', $.proxy( function( event )
      {
          if( TV.typeIsObject( typeof event.response.payload.errors ) )
          {
            for( /** @type {string} */ var fieldName in event.response.payload.errors )
            {
                this.formPanel.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
            }
          }
      }
      , this ) );
      
      cmd.execute();
    };
    
    //-------------------------------------------------------------------------
    
    return tsa.cmd.handlers.ScopesettingsHandler;
  }
);