/*
 * IBM Technical Support Appliance Version 2 Release 7 Modification 0
 *
 * (C) Copyright IBM Corp. 2018, 2020
 */

//-----------------------------------------------------------------------------

define(
  [ 'jquery'
  , 'tsa/TypeVerificator'
  , 'tsa/cmd/handlers/AbstractHandler'
  , 'tsa/widgets/ContentArea'
  , 'tsa/widgets/FormPanel'
  , 'tsa/widgets/TablePanel'
  , 'tsa/widgets/ButtonFactory'
  , 'tsa/cmd/Command'
  ],

  function( $
          , TV
          , AbstractHandler
          , ContentArea
          , FormPanel
          , TablePanel
          , ButtonFactory
          , Command
          )
  {
    //-------------------------------------------------------------------------
    
    var logger = tsa.getLogger( 'SecurityHandler' );
    
    //-------------------------------------------------------------------------
    
    /** @constructor
     *  @extends tsa.cmd.AbstractHandler
     *  Parameters added by this class (in addition to super-class' parameters):
     *    none.
     *    
     *  @param {Object} parameters The parameters. */
    
    tsa.cmd.handlers.SecurityHandler =  function( parameters )
    {
      if( !TV.typeIsObject( typeof parameters ) || ( parameters === null ) ) parameters = /** @type {?} */ ( {} );
      
      // Calling the constructor of the super-class...
      AbstractHandler.prototype.constructor.call( this, parameters );
    };
    
    //-------------------------------------------------------------------------
    
    // Inheriting:
    tsa.cmd.handlers.SecurityHandler.prototype = new AbstractHandler( {} );
    /** @type {?} */ ( tsa.cmd.handlers.SecurityHandler.prototype ).constructor = tsa.cmd.handlers.SecurityHandler;
    
    //-------------------------------------------------------------------------
    /**
     * Show main screen.
     */
    tsa.cmd.handlers.SecurityHandler.prototype._index = function()
    {
      /** @type {Object}                  */ var i18n        	= tsa.i18n;
      /** @type {tsa.widgets.ContentArea} */ var contentArea 	= null;
      /** @type {tsa.widgets.FormPanel}   */ var timeoutForm	= null;
      /** @type {tsa.widgets.FormPanel}   */ var maxPwdForm     = null;
      /** @type {tsa.cmd.Command}         */ var getInfoCmd		= new Command( { action : 'security/getInfo' } );
     
      
      contentArea = new ContentArea ( { title : i18n.labelSecurityHeading
    	  							  , helpDoc : 'sshelp_security_panel.html' } );
           
      contentArea.appendMessage( { text : i18n.labelSecurityCaption } );
      
      //need to manually create the required legend because the forms can't be link
      var legendP = document.createElement( 'p' );
      $( legendP ).addClass( 'tsa-required-legend-global' );
      
      var legendChunks = tsa.i18n.labelRequiredLegend.split( '*' );
      var legendSpan   = document.createElement( 'span' );
      $( legendSpan ).addClass( 'tsa-required-legend' );
      
      if( legendChunks.length === 2 ){
        $( legendSpan ).append( document.createTextNode( legendChunks[0] )
                              , $( '<span class="ibm-required"> * </span>' ).get(0)
                              , document.createTextNode( legendChunks[1] ) );
      }else{
        $( legendSpan ).text( tsa.i18n.labelRequiredLegend );
      }
      $( legendP ).append( legendSpan );
      
      contentArea.append( legendP );


 	 //TIME OUT
 	 timeoutForm = new FormPanel( { title  		: i18n.labelSecuritySessionTimeoutSettingsHeading
 		 						  , description : i18n.labelSecuritySessionTimeoutSettingsDesc
 		 						  , action 		: 'security/ajaxUpdateTimeout'
 		 						  , showReqHeader : false} );
 	 
 	 var timeoutDiv = document.createElement( 'div' );
 	 var timeoutTxtbox = document.createElement( 'input' );
 	 var timeoutChkbox = document.createElement( 'input' );
 	 var labelCheckInput = $("<label style='font-weight:normal'></label>").get(0);
	 
 	 timeoutTxtbox.type  = 'text';
 	 timeoutTxtbox.name  = 'timeout';
 	 $(timeoutTxtbox).attr('id', 'timeout');
 	
 	 timeoutChkbox.type  = 'checkbox';
 	 timeoutChkbox.name  = 'timeoutIsDisabled';
 	 $(timeoutChkbox).attr('id', 'timeoutCheckbox');
 
 	 $( labelCheckInput ).append( timeoutChkbox );
	 $( labelCheckInput ).append( i18n.sessionTimeoutDisable );
	 $( timeoutDiv ).append( labelCheckInput );
	 
	 timeoutForm.appendFieldRow( { label : i18n.labelSecurityDisableSessionTimeout
		 	  					 , element  : timeoutDiv
	 } );

	 timeoutForm.appendFieldRow( { label : i18n.labelSecuritySetSessionTimeout
		 		, required : true
		 		, hint	   : i18n.labelSecuritySetSessionTimeoutDesc
		 		, element  : timeoutTxtbox
	 } );

	 var hrDiv3 = document.createElement( 'div' );
	 $( hrDiv3 ).attr('class', 'ibm-rule');
	 $( hrDiv3 ).append ( document.createElement( 'hr' ) );
	 timeoutForm.appendJointRow( hrDiv3 );
	 
	 var timeoutButton = ButtonFactory.buildButton ({
	 type : 'arrow'
		  , label   : i18n.buttonSecurityChangeSessionTimeoutSettings
		  , onclick : function( event ){
			  timeoutForm.submit();
		  }
	 });

	 $( timeoutForm ).on( 'executionFailed', function( event )
	   {
		   if( TV.typeIsObject( typeof event.response.payload.errors ) )
		     {
		       for( var fieldName in event.response.payload.errors )
		        {
		       	  timeoutForm.showFieldError( fieldName, event.response.payload.errors[ fieldName ] );
		         }
		     }
		} );
	 
	 timeoutForm.appendJointRow( timeoutButton );
	 contentArea.append( timeoutForm.getDomNode() );
 	 
 	 //enable/disable textbox when checkbox is selected/deselected
	 var timeoutChanged = function( event )
     {
 	   var disableSessionTimeout = $( '#timeoutCheckbox' ).prop('checked');
 	   timeout.disabled = disableSessionTimeout;
 	   if(disableSessionTimeout)
 		   timeoutChkbox.value = 'disableSessionTimeout';
 	   else
 		  timeoutChkbox.value = '';
     }; 
     $( timeoutChkbox ).on( 'change', timeoutChanged );
 	  
     
     //MAX PASSWORD AGE
	 maxPwdForm = new FormPanel( { title 	   : i18n.labelMaximumPasswordAgeHeading
			  					 , description : i18n.labelMaximumPasswordAgeDesc
			  					 , action 	   : 'security/ajaxUpdatePwdAge'
			  					 , showReqHeader : false} );
	 
	 var maxAgeP = document.createElement( 'p' );
	 var maxAgeSelection = document.createElement( 'select' );
	 maxAgeSelection.name = 'maxPwdAge';
	 $( maxAgeSelection ).attr('id', 'maxPwdAge');
	 
	 var option = $('<option value="30">'+i18n.labelMaximumPasswordAgeDays30+'</option>').get(0);
	 $(maxAgeSelection).append( option );
	     
	 option = $('<option value="60">'+i18n.labelMaximumPasswordAgeDays60+'</option>').get(0);
     $(maxAgeSelection).append( option );
    
     option = $('<option value="90">'+i18n.labelMaximumPasswordAgeDays90+'</option>').get(0);
	 $(maxAgeSelection).append( option );
	  
	 maxPwdForm.appendFieldRow( { label : i18n.labelMaximumPasswordAge
	    	 	, required : true
	    	 	, element  : maxAgeSelection
	 } );
	     
	 var hrDiv4 = document.createElement( 'div' );
	 $( hrDiv4 ).attr('class', 'ibm-rule');
	 $( hrDiv4 ).append ( document.createElement( 'hr' ) );
	 maxPwdForm.appendJointRow( hrDiv4 );
	 
	 var pwdButton = ButtonFactory.buildButton ({
	 type : 'arrow'
  		  , label   : i18n.buttonMaximumPasswordAge
  		  , onclick : function( event ){
  			  maxPwdForm.submit();
  		  }
  	 });
	 maxPwdForm.appendJointRow( pwdButton );
   
	 contentArea.append( maxPwdForm.getDomNode() );
 	 
     $( getInfoCmd ).on( 'executionSucceeded', function( event ){
    	 
     	//Time out
     	var timeout = event.response.payload.timeout;
     	timeoutTxtbox.value = timeout;

     	if(event.response.payload.timeoutIsDisabled){
     		timeoutChkbox.checked = true;
     		timeoutTxtbox.disabled = true;
     	}else{
     		timeoutChkbox.checked = false;
     		timeoutTxtbox.disabled = false;
     	}

     	//Max password age
     	$(maxAgeSelection).val(event.response.payload.maxPwdAge);

      });

      // Get security info
      getInfoCmd.execute();

      this.show( contentArea.getDomNode() );
    };

    //-------------------------------------------------------------------------

    return tsa.cmd.handlers.SecurityHandler;
  }
);
